# Generated by Django 4.2.2 on 2023-07-05 19:12

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):
    dependencies = [
        ("diop", "0023_sitelog"),
    ]

    operations = [
        migrations.AlterField(
            model_name="sitelog",
            name="deliverygroup",
            field=models.ForeignKey(
                blank=True,
                null=True,
                on_delete=django.db.models.deletion.SET_NULL,
                to="diop.deliverygroup",
            ),
        ),
        migrations.AlterField(
            model_name="sitelog",
            name="machine",
            field=models.ForeignKey(
                blank=True,
                null=True,
                on_delete=django.db.models.deletion.SET_NULL,
                to="diop.machine",
            ),
        ),
        migrations.AlterField(
            model_name="sitelog",
            name="notification",
            field=models.ForeignKey(
                blank=True,
                null=True,
                on_delete=django.db.models.deletion.SET_NULL,
                to="diop.notification",
            ),
        ),
        migrations.AlterField(
            model_name="sitelog",
            name="reservation",
            field=models.ForeignKey(
                blank=True,
                null=True,
                on_delete=django.db.models.deletion.SET_NULL,
                to="diop.reservation",
            ),
        ),
        migrations.AlterField(
            model_name="sitelog",
            name="session",
            field=models.ForeignKey(
                blank=True,
                null=True,
                on_delete=django.db.models.deletion.SET_NULL,
                to="diop.session",
            ),
        ),
        migrations.AlterField(
            model_name="sitelog",
            name="task",
            field=models.ForeignKey(
                blank=True,
                null=True,
                on_delete=django.db.models.deletion.SET_NULL,
                to="diop.task",
            ),
        ),
        migrations.AlterField(
            model_name="sitelog",
            name="user",
            field=models.ForeignKey(
                blank=True,
                null=True,
                on_delete=django.db.models.deletion.SET_NULL,
                to="diop.user",
            ),
        ),
    ]
