# Generated by Django 4.2.1 on 2023-05-23 16:32

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("diop", "0013_alter_issue_description_alter_issue_resolved_and_more"),
    ]

    operations = [
        migrations.AddField(
            model_name="session",
            name="updated",
            field=models.BooleanField(
                default=True,
                help_text="Flag tracking if the session status is up-to-date. Has to be set to False by every function updating an entry and only re-set to True once the update has completed successfully.",
                verbose_name="Updated",
            ),
        ),
    ]
