# Generated by Django 4.1.7 on 2023-04-17 12:38

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("diop", "0006_alter_deliverygroup_booking_required_and_more"),
    ]

    operations = [
        migrations.AddField(
            model_name="deliverygroup",
            name="access_users",
            field=models.ManyToManyField(to="diop.user"),
        ),
    ]
