# Generated by Django 4.1.7 on 2023-04-14 10:04

from django.db import migrations, models


class Migration(migrations.Migration):
    initial = True

    dependencies = []

    operations = [
        migrations.CreateModel(
            name="SiteConfig",
            fields=[
                (
                    "setting",
                    models.CharField(max_length=64, primary_key=True, serialize=False),
                ),
                ("value", models.CharField(blank=True, max_length=512)),
            ],
        ),
        migrations.CreateModel(
            name="User",
            fields=[
                (
                    "username",
                    models.CharField(max_length=32, primary_key=True, serialize=False),
                ),
                ("fullname", models.CharField(max_length=128)),
                ("groupname", models.CharField(max_length=128)),
                ("department", models.CharField(max_length=128)),
                ("email", models.CharField(max_length=128)),
                ("ppms_group", models.CharField(max_length=64)),
                ("ppms_fullname", models.CharField(max_length=128)),
                ("gracetime", models.IntegerField(default=0)),
                ("active", models.BooleanField(default=True)),
                ("accessgroup_member", models.BooleanField(default=False)),
            ],
        ),
    ]
