# Generated by Django 4.2.2 on 2023-06-13 21:50

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("diop", "0017_alter_machine_deliverygroup_alter_machine_state"),
    ]

    operations = [
        migrations.AddField(
            model_name="machine",
            name="req_maintenance",
            field=models.BooleanField(
                default=False,
                help_text="Enter maintenance mode as soon as the machine is free.",
                verbose_name="Request Maintenance",
            ),
        ),
    ]
