# Generated by Django 4.1.7 on 2023-04-14 11:40

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("diop", "0002_machine"),
    ]

    operations = [
        migrations.AlterModelOptions(
            name="siteconfig",
            options={"verbose_name": "Site Config"},
        ),
        migrations.AlterField(
            model_name="machine",
            name="fqdn",
            field=models.CharField(
                max_length=64,
                primary_key=True,
                serialize=False,
                verbose_name="Fully Qualified Domain Name",
            ),
        ),
        migrations.AlterField(
            model_name="machine",
            name="maintenance",
            field=models.BooleanField(default=False, verbose_name="Maintenance Mode"),
        ),
        migrations.AlterField(
            model_name="machine",
            name="powerstate",
            field=models.CharField(max_length=16, verbose_name="Power State"),
        ),
        migrations.AlterField(
            model_name="machine",
            name="registration",
            field=models.CharField(max_length=16, verbose_name="Registration State"),
        ),
        migrations.AlterField(
            model_name="machine",
            name="state",
            field=models.CharField(max_length=16, verbose_name="Machine State"),
        ),
    ]
