# Generated by Django 4.2.2 on 2023-06-21 12:31

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):
    dependencies = [
        ("diop", "0020_reservation_ppmsbooking_and_more"),
    ]

    operations = [
        migrations.AlterField(
            model_name="issue",
            name="description",
            field=models.TextField(),
        ),
        migrations.AlterField(
            model_name="issue",
            name="task",
            field=models.ForeignKey(
                blank=True,
                help_text="Related task causing the issue (if applicable).",
                null=True,
                on_delete=django.db.models.deletion.CASCADE,
                to="diop.task",
            ),
        ),
        migrations.AlterField(
            model_name="issue",
            name="user",
            field=models.ForeignKey(
                blank=True,
                help_text="Related user (if applicable).",
                null=True,
                on_delete=django.db.models.deletion.CASCADE,
                to="diop.user",
            ),
        ),
    ]
