# Generated by Django 4.2.1 on 2023-06-02 18:48

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):
    dependencies = [
        ("diop", "0014_session_updated"),
    ]

    operations = [
        migrations.RenameField(
            model_name="session",
            old_name="fqdn",
            new_name="machine",
        ),
        migrations.AlterField(
            model_name="session",
            name="machine",
            field=models.ForeignKey(
                help_text="Machine associated to the session",
                on_delete=django.db.models.deletion.CASCADE,
                to="diop.machine",
                verbose_name="Machine",
            ),
        ),
    ]
