{% extends "base.html" %}


{% block title %}
    <title>{{ request.site.name }} Machines</title>
{% endblock title %}


{% block content %}
    <h2>{{ request.site.name }} Machines</h2>

    <table class="table table-striped table-hover table-sm">
        <thead>
            <th>Name</th>
            <th>Category</th>
            <th>State</th>
            <th>Maintenance</th>
        </thead>
        {% for machine in machines %}
            <tr class="position-relative {{ machine.tr_class }}">
                <td>{{ machine.hostname }}</td>
                <td>{{ machine.deliverygroup }}</td>
                <td>{{ machine.state }}</td>
                <td>{{ machine.maintenance }}</td>
            </tr>
        {% endfor %}
    </table>
{% endblock content %}
