# Generated by Django 4.2.2 on 2023-07-07 08:31

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("diop", "0025_alter_user_department_alter_user_fullname_and_more"),
    ]

    operations = [
        migrations.RemoveField(
            model_name="user",
            name="active",
        ),
        migrations.AddField(
            model_name="user",
            name="enabled",
            field=models.BooleanField(
                default=True,
                help_text="Account is enabled in AD (2nd bit of `userAccountControl` is 0).",
            ),
        ),
    ]
