# Generated by Django 4.2.2 on 2023-06-21 12:35

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("diop", "0021_alter_issue_description_alter_issue_task_and_more"),
    ]

    operations = [
        migrations.AlterField(
            model_name="notification",
            name="body",
            field=models.TextField(verbose_name="The notification body / content."),
        ),
        migrations.AlterField(
            model_name="task",
            name="description",
            field=models.TextField(blank=True, default=""),
            preserve_default=False,
        ),
    ]
