# Generated by Django 4.2.2 on 2023-07-11 20:03

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("diop", "0028_deliverygroup_unique_session"),
    ]

    operations = [
        migrations.AlterField(
            model_name="deliverygroup",
            name="access_users",
            field=models.ManyToManyField(
                blank=True,
                help_text="Users that are explicitly granted access to this DG.",
                to="diop.user",
            ),
        ),
        migrations.AlterField(
            model_name="deliverygroup",
            name="host_prefix",
            field=models.CharField(
                blank=True,
                default="",
                help_text="Hostnames starting with this prefix will be considered members of the DG.",
                max_length=32,
                verbose_name="Hostname Prefix",
            ),
        ),
    ]
