# Generated by Django 4.2.2 on 2023-07-06 20:55

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("diop", "0024_alter_sitelog_deliverygroup_alter_sitelog_machine_and_more"),
    ]

    operations = [
        migrations.AlterField(
            model_name="user",
            name="department",
            field=models.CharField(
                blank=True,
                help_text='LDAP attribute "department" (may be set manually but LDAP has priority in case the field is present there).',
                max_length=128,
                null=True,
            ),
        ),
        migrations.AlterField(
            model_name="user",
            name="fullname",
            field=models.CharField(
                blank=True,
                help_text='LDAP attribute "displayName".',
                max_length=128,
                null=True,
                verbose_name="Full Name",
            ),
        ),
        migrations.AlterField(
            model_name="user",
            name="groupname",
            field=models.CharField(
                blank=True,
                help_text="Manually set (cannot be derived reliably via LDAP).",
                max_length=128,
                null=True,
                verbose_name="Group Name",
            ),
        ),
        migrations.AlterField(
            model_name="user",
            name="ppms_fullname",
            field=models.CharField(
                blank=True, max_length=128, null=True, verbose_name="PPMS Full Name"
            ),
        ),
        migrations.AlterField(
            model_name="user",
            name="ppms_group",
            field=models.CharField(
                blank=True, max_length=64, null=True, verbose_name="PPMS Group"
            ),
        ),
    ]
