# Generated by Django 4.1.7 on 2023-05-22 10:10

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("diop", "0011_task_issue"),
    ]

    operations = [
        migrations.AddField(
            model_name="machine",
            name="active",
            field=models.BooleanField(
                default=True,
                help_text="Indicating if the machine is active or has been retired. Entries having this set to False will be ignored by update task completeness checks.",
                verbose_name="Active",
            ),
        ),
        migrations.AddField(
            model_name="machine",
            name="updated",
            field=models.BooleanField(
                default=True,
                help_text="Flag tracking if the machine status is up-to-date. Has to be set to False by every function updating an entry and only re-set to True once the update has completed successfully.",
                verbose_name="Updated",
            ),
        ),
    ]
