# Generated by Django 4.2.2 on 2023-06-13 20:20

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):
    dependencies = [
        ("diop", "0016_notification"),
    ]

    operations = [
        migrations.AlterField(
            model_name="machine",
            name="deliverygroup",
            field=models.ForeignKey(
                help_text="Delivery Group the machine belongs to.",
                on_delete=django.db.models.deletion.CASCADE,
                to="diop.deliverygroup",
                verbose_name="Delivery Group",
            ),
        ),
        migrations.AlterField(
            model_name="machine",
            name="state",
            field=models.CharField(
                help_text="Current machine status.",
                max_length=16,
                verbose_name="Machine State",
            ),
        ),
    ]
