# Generated by Django 4.2.2 on 2023-07-07 21:57

from django.db import migrations


class Migration(migrations.Migration):
    dependencies = [
        ("diop", "0026_remove_user_active_user_enabled"),
    ]

    operations = [
        migrations.DeleteModel(
            name="SiteConfig",
        ),
    ]
