# Generated by Django 4.1.7 on 2023-05-15 20:29

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("diop", "0009_alter_session_user"),
    ]

    operations = [
        migrations.AddField(
            model_name="machine",
            name="agent",
            field=models.CharField(
                max_length=32, null=True, verbose_name="Agent Version"
            ),
        ),
    ]
