# Generated by Django 4.1.7 on 2023-04-14 14:04

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):
    dependencies = [
        ("diop", "0004_deliverygroup"),
    ]

    operations = [
        migrations.AddField(
            model_name="machine",
            name="deliverygroup",
            field=models.ForeignKey(
                default="Dinner",
                on_delete=django.db.models.deletion.CASCADE,
                to="diop.deliverygroup",
            ),
            preserve_default=False,
        ),
    ]
