# Generated by Django 4.2.2 on 2023-06-16 18:53

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("diop", "0018_machine_req_maintenance"),
    ]

    operations = [
        migrations.AddField(
            model_name="user",
            name="disconnected_max",
            field=models.IntegerField(
                default=0,
                help_text="Hours after which a disconnected session is terminated, negative values mean no session termination will be enforced.",
                verbose_name="Max. disconnected hours.",
            ),
        ),
        migrations.AlterField(
            model_name="user",
            name="accessgroup_member",
            field=models.BooleanField(
                default=False,
                help_text="User is in the AD-group permitting platform access.",
            ),
        ),
        migrations.AlterField(
            model_name="user",
            name="fullname",
            field=models.CharField(max_length=128, verbose_name="Full Name"),
        ),
        migrations.AlterField(
            model_name="user",
            name="gracetime",
            field=models.IntegerField(
                default=0, help_text="Number of days until data expires on the storage."
            ),
        ),
        migrations.AlterField(
            model_name="user",
            name="groupname",
            field=models.CharField(max_length=128, verbose_name="Group Name"),
        ),
        migrations.AlterField(
            model_name="user",
            name="ppms_fullname",
            field=models.CharField(max_length=128, verbose_name="PPMS Full Name"),
        ),
        migrations.AlterField(
            model_name="user",
            name="ppms_group",
            field=models.CharField(max_length=64, verbose_name="PPMS Group"),
        ),
    ]
