# Generated by Django 4.2.2 on 2023-07-11 19:55

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("diop", "0027_delete_siteconfig"),
    ]

    operations = [
        migrations.AddField(
            model_name="deliverygroup",
            name="unique_session",
            field=models.BooleanField(
                default=True,
                help_text="Allow only one session per user across *ALL* groups having this flag enabled (enforced via a housekeeping task).",
            ),
        ),
    ]
