# Generated by Django 4.1.7 on 2023-04-17 15:43

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):
    dependencies = [
        ("diop", "0008_session_sessionproperties"),
    ]

    operations = [
        migrations.AlterField(
            model_name="session",
            name="user",
            field=models.ForeignKey(
                blank=True,
                help_text="User associated to the session, can be empty!",
                null=True,
                on_delete=django.db.models.deletion.CASCADE,
                to="diop.user",
                verbose_name="User",
            ),
        ),
    ]
