# Generated by Django 4.1.7 on 2023-05-22 12:04

from django.db import migrations, models
import django.utils.timezone


class Migration(migrations.Migration):
    dependencies = [
        ("diop", "0012_machine_active_machine_updated"),
    ]

    operations = [
        migrations.AlterField(
            model_name="issue",
            name="description",
            field=models.CharField(max_length=4096),
        ),
        migrations.AlterField(
            model_name="issue",
            name="resolved",
            field=models.BooleanField(
                default=False, help_text="Flag indicating the issue is resolved."
            ),
        ),
        migrations.AlterField(
            model_name="issue",
            name="severity",
            field=models.SmallIntegerField(
                choices=[
                    (0, "Low"),
                    (1, "Medium"),
                    (2, "High"),
                    (3, "Critical"),
                    (4, "Meltdown"),
                ],
                default=0,
                help_text="Values from 0 to 4, higher means more critical.",
            ),
        ),
        migrations.AlterField(
            model_name="issue",
            name="timestamp",
            field=models.DateTimeField(
                auto_now_add=True, help_text="Time the issue occurred."
            ),
        ),
        migrations.AlterField(
            model_name="task",
            name="t_start",
            field=models.DateTimeField(
                default=django.utils.timezone.now, verbose_name="Start Time"
            ),
        ),
    ]
