<div class="btn-group btn-group-sm mx-4" role="group">
    <a href="/admin/diop/issue/?resolved__exact=0"
       class="btn btn-block btn-outline-danger p-1"
       aria-current="page">Issues</a>
    <a href="/admin/diop/issue/?resolved__exact=0"
       class="btn btn-block btn-danger p-1 px-2">{{ issues_unresolved_count }}</a>
</div>
