# Generated by Django 4.1.7 on 2023-04-14 10:43

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("diop", "0001_initial"),
    ]

    operations = [
        migrations.CreateModel(
            name="Machine",
            fields=[
                (
                    "fqdn",
                    models.CharField(max_length=64, primary_key=True, serialize=False),
                ),
                ("state", models.CharField(max_length=16)),
                ("powerstate", models.CharField(max_length=16)),
                ("maintenance", models.BooleanField(default=False)),
                ("registration", models.CharField(max_length=16)),
            ],
        ),
    ]
