# Generated by Django 4.1.7 on 2023-04-14 13:46

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("diop", "0003_alter_siteconfig_options_alter_machine_fqdn_and_more"),
    ]

    operations = [
        migrations.CreateModel(
            name="DeliveryGroup",
            fields=[
                (
                    "dg_name",
                    models.CharField(
                        max_length=32,
                        primary_key=True,
                        serialize=False,
                        verbose_name="Name of the Delivery Group (DG)",
                    ),
                ),
                (
                    "host_prefix",
                    models.CharField(
                        max_length=32, verbose_name="Hostname prefix identifying the DG"
                    ),
                ),
                (
                    "booking_required",
                    models.BooleanField(
                        default=False, verbose_name="DG requires booking"
                    ),
                ),
            ],
            options={
                "verbose_name": "Delivery Group",
            },
        ),
    ]
