.sidebar-nav {
    margin-top: 20px;
    padding: 0;
    list-style: none;
}

a {
    text-decoration: none;
}

.message-style-icon {
    font-size: 2rem;
    padding: 2rem;
    padding-top: 0rem;
    padding-bottom: 0rem;
}