# Generated by Django 4.2.2 on 2023-07-11 21:56

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("diop", "0029_alter_deliverygroup_access_users_and_more"),
    ]

    operations = [
        migrations.CreateModel(
            name="MessageTemplate",
            fields=[
                (
                    "name",
                    models.CharField(
                        max_length=64,
                        primary_key=True,
                        serialize=False,
                        verbose_name="Template Name",
                    ),
                ),
                (
                    "category",
                    models.CharField(
                        choices=[
                            ("Q", "Question"),
                            ("I", "Info"),
                            ("W", "Warning"),
                            ("E", "Error"),
                        ],
                        default="I",
                        max_length=1,
                        verbose_name="Message Category",
                    ),
                ),
                (
                    "subject",
                    models.CharField(
                        help_text="The template subject / title.", max_length=256
                    ),
                ),
                ("body", models.TextField(help_text="The template body / content.")),
            ],
            options={
                "verbose_name": "Message Template",
            },
        ),
    ]
