# Generated by Django 4.1.7 on 2023-04-17 12:29

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):
    dependencies = [
        ("diop", "0005_machine_deliverygroup"),
    ]

    operations = [
        migrations.AlterField(
            model_name="deliverygroup",
            name="booking_required",
            field=models.BooleanField(
                default=False,
                help_text="Sessions in this DG require a valid booking.",
                verbose_name="Booking Required",
            ),
        ),
        migrations.AlterField(
            model_name="deliverygroup",
            name="dg_name",
            field=models.CharField(
                help_text="Name of the Delivery Group (DG).",
                max_length=32,
                primary_key=True,
                serialize=False,
                verbose_name="Group Name",
            ),
        ),
        migrations.AlterField(
            model_name="deliverygroup",
            name="host_prefix",
            field=models.CharField(
                help_text="Hostnames starting with this prefix will be considered members of the DG.",
                max_length=32,
                verbose_name="Hostname Prefix",
            ),
        ),
        migrations.AlterField(
            model_name="machine",
            name="deliverygroup",
            field=models.ForeignKey(
                on_delete=django.db.models.deletion.CASCADE,
                to="diop.deliverygroup",
                verbose_name="Delivery Group",
            ),
        ),
        migrations.AlterField(
            model_name="machine",
            name="fqdn",
            field=models.CharField(
                help_text="Fully Qualified Domain Name",
                max_length=64,
                primary_key=True,
                serialize=False,
                verbose_name="FQDN",
            ),
        ),
    ]
